from datetime import datetime
from typing import Any, ClassVar, Dict, List, Optional, Type, TypeVar
from uuid import UUID

from google.protobuf.json_format import MessageToDict
from google.protobuf.message import Message
from pydantic import UUID4, BaseModel, validate_call

from admac.grpc.accounts_pb2 import WalletPackageRetrieveRequest
from admac.schemas import (
    EducationalExperience,
    Goal,
    Profile,
    Skill,
    SocialMedia,
    Suggestion,
    WalletPackage,
    WalletPackages,
    WorkExperience,
)
from admac.schemas import User as UserSchema
from admac.schemas import Wallet as WalletSchema

from . import config
from .api_constants import SUCCESS_CODE, RequestMessages, RPCMethodNames, Stubs

M = TypeVar("M", bound=BaseModel)


class ValidationError(Exception):
    pass


class ResponseCodeError(Exception):
    pass


class RPCClient(BaseModel):
    _instances: ClassVar[dict] = {}
    stub_cls: ClassVar

    def __new__(cls, *args, **kwargs):
        # Ensure that each subclass has its own singleton instance
        if cls not in cls._instances:
            cls._instances[cls] = super().__new__(cls)
        return cls._instances[cls]

    @validate_call(config=dict(arbitrary_types_allowed=True))
    async def perform_rpc(
        self,
        *,
        rpc_method_name: str,
        request_message: Optional[Message] = None,
        jwt: Optional[str] = None,
        response_model: Optional[Type[M]] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ) -> list[M] | dict:
        """Perform the remote-precedure-call.

        Parameters:
            rpc_method_name: The name of the RPC method (e.g., `Retrieve`).
            request_message: The request message instance.
            timeout: RPC timeout.
            wait_for_ready: Wait for Channel to be ready in case is not.
        """
        if config.CHANNEL is None:
            await config.construct_channel()

        RPCMethod = getattr(self.stub_cls(config.CHANNEL), rpc_method_name, None)
        if not RPCMethod:
            raise ValidationError("Invalid RPC method name.")

        metadata = [("api-key", config.grpc_settings.API_KEY)]
        if request_id:
            metadata.append(("x-request-id", str(request_id)))
        if jwt:
            metadata.append(("authorization", jwt))
        response = await RPCMethod(
            request_message,
            metadata=metadata,
            timeout=config.grpc_settings.TIMEOUT if timeout is None else timeout,
            wait_for_ready=config.grpc_settings.WAIT_FOR_READY
            if wait_for_ready is None
            else wait_for_ready,
        )

        # Raise error if response is not successful
        if response.code != SUCCESS_CODE:
            raise ResponseCodeError(response.code)

        response_dict = MessageToDict(response, preserving_proto_field_name=True)
        data = response_dict.get("data", [])

        if not response_model:
            return data

        return [response_model.model_validate(item) for item in data]


class User(RPCClient):
    stub_cls = Stubs.USER

    @validate_call
    async def fetch_bulk(
        self,
        identifiers: List[str | UUID4],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch users' info by identifier."""
        request = RequestMessages.FETCH_BULK_USER(uuid_id=map(str, identifiers))
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_BULK,
            request_message=request,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_one_by_id(
        self,
        identifier: str | UUID4,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch a user's info using user identifier."""
        request = RequestMessages.FETCH_USER(uuid_id=str(identifier))
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_message=request,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_one_by_jwt(
        self,
        jwt: str,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch a user's info using user jwt."""
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            jwt=jwt,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def update_one_by_id(
        self,
        identifier: str | UUID4,
        user: Optional[UserSchema] = None,
        profile: Optional[Profile] = None,
        work_experience: Optional[WorkExperience] = None,
        skill: Optional[Skill] = None,
        educational_experience: Optional[EducationalExperience] = None,
        goal: Optional[Goal] = None,
        suggestion: Optional[Suggestion] = None,
        social_media: Optional[SocialMedia] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Update a user's info using user identifier."""

        # Prepare request data
        request: Dict[str, Any] = {"uuid_id": str(identifier)}

        if user:
            request.update(user.model_dump())  # main user fields

        nested_mapping = {
            "profile": (profile, RequestMessages.PROFILE),
            "work_experience": (work_experience, RequestMessages.WORK_EXPERIENCE),
            "skill": (skill, RequestMessages.SKILL),
            "educational_experience": (
                educational_experience,
                RequestMessages.EDUCATIONAL_EXPERIENCE,
            ),
            "goal": (goal, RequestMessages.GOAL),
            "suggestion": (suggestion, RequestMessages.SUGGESTION),
            "social_media": (social_media, RequestMessages.SOCIAL_MEDIA),
        }

        for field_name, (data, request_class) in nested_mapping.items():
            request[field_name] = request_class(**data.model_dump()) if data else None

        request = RequestMessages.UPDATE_USER(**request)

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.UPDATE_ONE,
            request_message=request,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def update_one_by_jwt(
        self,
        jwt: str,
        user: Optional[UserSchema] = None,
        profile: Optional[Profile] = None,
        work_experience: Optional[WorkExperience] = None,
        skill: Optional[Skill] = None,
        educational_experience: Optional[EducationalExperience] = None,
        goal: Optional[Goal] = None,
        suggestion: Optional[Suggestion] = None,
        social_media: Optional[SocialMedia] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Update a user's info using JWT with nested validated data."""

        # Prepare request data
        request = {}
        if user:
            request.update(user.model_dump())  # main user fields

        nested_mapping = {
            "profile": (profile, RequestMessages.PROFILE),
            "work_experience": (work_experience, RequestMessages.WORK_EXPERIENCE),
            "skill": (skill, RequestMessages.SKILL),
            "educational_experience": (
                educational_experience,
                RequestMessages.EDUCATIONAL_EXPERIENCE,
            ),
            "goal": (goal, RequestMessages.GOAL),
            "suggestion": (suggestion, RequestMessages.SUGGESTION),
            "social_media": (social_media, RequestMessages.SOCIAL_MEDIA),
        }

        for field_name, (data, request_class) in nested_mapping.items():
            request[field_name] = request_class(**data.model_dump()) if data else None

        # Create gRPC request
        request = RequestMessages.UPDATE_USER(**request)

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.UPDATE_ONE,
            request_message=request,
            jwt=jwt,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )


class Asset(RPCClient):
    stub_cls = Stubs.ASSET

    @validate_call
    async def create(
        self,
        owner_id: str,
        asset_type: str,
        asset_id: str,
        asset_name: str,
        expire_date: Optional[str | datetime] = None,
        is_active: Optional[bool] = None,
        installment_identifier: Optional[str] = None,
        registration_completed: Optional[bool] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Create an Asset for a User."""

        # Fill the protobuf request
        request = RequestMessages.CREATE_ASSET(
            owner_id=owner_id, type=asset_type, asset_id=asset_id, asset_name=asset_name
        )
        if expire_date:
            request.expire_date = str(expire_date)
        if is_active is not None:
            request.is_active = is_active
        if registration_completed is not None:
            request.registration_completed = registration_completed
        if installment_identifier:
            request.installment_identifier = installment_identifier

        # Perform grpc
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.CREATE,
            request_message=request,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_all_by_id(
        self,
        owner_id: str | UUID,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch user assets by id."""

        # Fill the protobuf request
        request = RequestMessages.LIST_ASSET(owner_id=str(owner_id))

        # Perform grpc
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ALL,
            request_message=request,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_all_by_jwt(
        self,
        jwt: str,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch user assets by jwt."""

        # Perform grpc
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ALL,
            jwt=jwt,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )


class Wallet(RPCClient):
    stub_cls = Stubs.WALLET

    @validate_call
    async def fetch_one_by_owner_id(
        self,
        owner_id: str | UUID,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch user wallet by user id."""

        # Fill the protobuf request
        request = RequestMessages.FETCH_WALLET(owner_id=str(owner_id))

        # Perform grpc
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_message=request,
            response_model=WalletSchema,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_one_by_owner_jwt(
        self,
        jwt: str,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch user wallet by user jwt."""

        # Perform grpc
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            jwt=jwt,
            response_model=WalletSchema,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_one_package(
        self,
        package_name: str,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch one wallet package by its name."""

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE_PACKAGE,
            request_message=WalletPackageRetrieveRequest(name=package_name),
            response_model=WalletPackage,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_all_packages(
        self,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch all wallet packages."""

        # Perform grpc
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ALL_PACKAGES,
            response_model=WalletPackages,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def topup(
        self,
        wallet_id: str | UUID,
        amount: Optional[int] = None,
        package_name: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Add credit to a wallet."""
        request = RequestMessages.TOPUP_WALLET(uuid_id=str(wallet_id))
        if amount:
            request.amount = amount
        if package_name:
            request.package_name = package_name

        # Perform grpc
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.TOPUP,
            request_message=request,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def spend(
        self,
        wallet_id: str | UUID,
        amount: int,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Spend amount from wallet a wallet."""
        request = RequestMessages.SPEND_WALLET(uuid_id=str(wallet_id), amount=amount)
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.SPEND,
            request_message=request,
            timeout=timeout,
            request_id=request_id,
            wait_for_ready=wait_for_ready,
        )



from admac.grpc.accounts_pb2 import (
    AssetCreateRequest,
    EducationalExperienceRequest,
    GoalRequest,
    ListAssetRequest,
    ProfileRequest,
    SkillRequest,
    SocialMediaRequest,
    SuggestionRequest,
    UserBulkRetrieveRequest,
    UserRetrieveRequest,
    UserUpdateRequest,
    WalletRetrieveRequest,
    WalletSpendRequest,
    WalletTopupRequest,
    WorkExperienceRequest,
)
from admac.grpc.accounts_pb2_grpc import (
    AssetControllerStub,
    UserControllerStub,
    WalletControllerStub,
)

SUCCESS_CODE = "0"


def _singletonize_class_vars(cls):
    for attr_name, attr_value in cls.__dict__.items():
        if isinstance(attr_value, type):  # it's a class
            orig_cls = attr_value

            class SingletonWrapper(orig_cls):
                _instances = {}

                def __new__(cls, *args, **kwargs):
                    if cls not in cls._instances:
                        cls._instances[cls] = super().__new__(cls)
                    return cls._instances[cls]

            setattr(cls, attr_name, SingletonWrapper)
    return cls


class RequestMessages:
    # Main
    FETCH_USER = UserRetrieveRequest
    FETCH_BULK_USER = UserBulkRetrieveRequest
    UPDATE_USER = UserUpdateRequest
    CREATE_ASSET = AssetCreateRequest
    LIST_ASSET = ListAssetRequest
    FETCH_WALLET = WalletRetrieveRequest
    TOPUP_WALLET = WalletTopupRequest
    SPEND_WALLET = WalletSpendRequest

    # Side
    PROFILE = ProfileRequest
    WORK_EXPERIENCE = WorkExperienceRequest
    SKILL = SkillRequest
    EDUCATIONAL_EXPERIENCE = EducationalExperienceRequest
    GOAL = GoalRequest
    SUGGESTION = SuggestionRequest
    SOCIAL_MEDIA = SocialMediaRequest


class RPCMethodNames:
    FETCH_ONE = "Retrieve"
    FETCH_BULK = "BulkRetrieve"
    FETCH_ALL = "List"
    FETCH_ONE_PACKAGE = "RetrievePackage"
    FETCH_ALL_PACKAGES = "ListPackages"
    SPEND = "Spend"
    TOPUP = "Topup"
    UPDATE_ONE = "Update"
    CREATE = "Create"


@_singletonize_class_vars
class Stubs:
    USER = UserControllerStub
    ASSET = AssetControllerStub
    WALLET = WalletControllerStub



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import accounts_pb2 as accounts_dot_grpc_dot_accounts__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


class AssetControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Create = channel.unary_unary(
                '/darsman.accounts.AssetController/Create',
                request_serializer=accounts_dot_grpc_dot_accounts__pb2.AssetCreateRequest.SerializeToString,
                response_deserializer=accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
                )
        self.List = channel.unary_unary(
                '/darsman.accounts.AssetController/List',
                request_serializer=accounts_dot_grpc_dot_accounts__pb2.ListAssetRequest.SerializeToString,
                response_deserializer=accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
                )


class AssetControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AssetControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=accounts_dot_grpc_dot_accounts__pb2.AssetCreateRequest.FromString,
                    response_serializer=accounts_dot_grpc_dot_accounts__pb2.Response.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=accounts_dot_grpc_dot_accounts__pb2.ListAssetRequest.FromString,
                    response_serializer=accounts_dot_grpc_dot_accounts__pb2.Response.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.accounts.AssetController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AssetController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.accounts.AssetController/Create',
            accounts_dot_grpc_dot_accounts__pb2.AssetCreateRequest.SerializeToString,
            accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.accounts.AssetController/List',
            accounts_dot_grpc_dot_accounts__pb2.ListAssetRequest.SerializeToString,
            accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class UserControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.BulkRetrieve = channel.unary_unary(
                '/darsman.accounts.UserController/BulkRetrieve',
                request_serializer=accounts_dot_grpc_dot_accounts__pb2.UserBulkRetrieveRequest.SerializeToString,
                response_deserializer=accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
                )
        self.Retrieve = channel.unary_unary(
                '/darsman.accounts.UserController/Retrieve',
                request_serializer=accounts_dot_grpc_dot_accounts__pb2.UserRetrieveRequest.SerializeToString,
                response_deserializer=accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
                )
        self.Update = channel.unary_unary(
                '/darsman.accounts.UserController/Update',
                request_serializer=accounts_dot_grpc_dot_accounts__pb2.UserUpdateRequest.SerializeToString,
                response_deserializer=accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
                )


class UserControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def BulkRetrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Update(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_UserControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'BulkRetrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.BulkRetrieve,
                    request_deserializer=accounts_dot_grpc_dot_accounts__pb2.UserBulkRetrieveRequest.FromString,
                    response_serializer=accounts_dot_grpc_dot_accounts__pb2.Response.SerializeToString,
            ),
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=accounts_dot_grpc_dot_accounts__pb2.UserRetrieveRequest.FromString,
                    response_serializer=accounts_dot_grpc_dot_accounts__pb2.Response.SerializeToString,
            ),
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=accounts_dot_grpc_dot_accounts__pb2.UserUpdateRequest.FromString,
                    response_serializer=accounts_dot_grpc_dot_accounts__pb2.Response.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.accounts.UserController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class UserController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def BulkRetrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.accounts.UserController/BulkRetrieve',
            accounts_dot_grpc_dot_accounts__pb2.UserBulkRetrieveRequest.SerializeToString,
            accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.accounts.UserController/Retrieve',
            accounts_dot_grpc_dot_accounts__pb2.UserRetrieveRequest.SerializeToString,
            accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.accounts.UserController/Update',
            accounts_dot_grpc_dot_accounts__pb2.UserUpdateRequest.SerializeToString,
            accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class WalletControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListPackages = channel.unary_unary(
                '/darsman.accounts.WalletController/ListPackages',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
                )
        self.Retrieve = channel.unary_unary(
                '/darsman.accounts.WalletController/Retrieve',
                request_serializer=accounts_dot_grpc_dot_accounts__pb2.WalletRetrieveRequest.SerializeToString,
                response_deserializer=accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
                )
        self.RetrievePackage = channel.unary_unary(
                '/darsman.accounts.WalletController/RetrievePackage',
                request_serializer=accounts_dot_grpc_dot_accounts__pb2.WalletPackageRetrieveRequest.SerializeToString,
                response_deserializer=accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
                )
        self.Spend = channel.unary_unary(
                '/darsman.accounts.WalletController/Spend',
                request_serializer=accounts_dot_grpc_dot_accounts__pb2.WalletSpendRequest.SerializeToString,
                response_deserializer=accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
                )
        self.Topup = channel.unary_unary(
                '/darsman.accounts.WalletController/Topup',
                request_serializer=accounts_dot_grpc_dot_accounts__pb2.WalletTopupRequest.SerializeToString,
                response_deserializer=accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
                )


class WalletControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ListPackages(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrievePackage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Spend(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Topup(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_WalletControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListPackages': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPackages,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=accounts_dot_grpc_dot_accounts__pb2.Response.SerializeToString,
            ),
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=accounts_dot_grpc_dot_accounts__pb2.WalletRetrieveRequest.FromString,
                    response_serializer=accounts_dot_grpc_dot_accounts__pb2.Response.SerializeToString,
            ),
            'RetrievePackage': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrievePackage,
                    request_deserializer=accounts_dot_grpc_dot_accounts__pb2.WalletPackageRetrieveRequest.FromString,
                    response_serializer=accounts_dot_grpc_dot_accounts__pb2.Response.SerializeToString,
            ),
            'Spend': grpc.unary_unary_rpc_method_handler(
                    servicer.Spend,
                    request_deserializer=accounts_dot_grpc_dot_accounts__pb2.WalletSpendRequest.FromString,
                    response_serializer=accounts_dot_grpc_dot_accounts__pb2.Response.SerializeToString,
            ),
            'Topup': grpc.unary_unary_rpc_method_handler(
                    servicer.Topup,
                    request_deserializer=accounts_dot_grpc_dot_accounts__pb2.WalletTopupRequest.FromString,
                    response_serializer=accounts_dot_grpc_dot_accounts__pb2.Response.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.accounts.WalletController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class WalletController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ListPackages(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.accounts.WalletController/ListPackages',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.accounts.WalletController/Retrieve',
            accounts_dot_grpc_dot_accounts__pb2.WalletRetrieveRequest.SerializeToString,
            accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrievePackage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.accounts.WalletController/RetrievePackage',
            accounts_dot_grpc_dot_accounts__pb2.WalletPackageRetrieveRequest.SerializeToString,
            accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Spend(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.accounts.WalletController/Spend',
            accounts_dot_grpc_dot_accounts__pb2.WalletSpendRequest.SerializeToString,
            accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Topup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.accounts.WalletController/Topup',
            accounts_dot_grpc_dot_accounts__pb2.WalletTopupRequest.SerializeToString,
            accounts_dot_grpc_dot_accounts__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from datetime import date, datetime
from typing import List, Literal, Optional

from pydantic import BaseModel, EmailStr, field_validator, model_validator


class User(BaseModel):
    email: Optional[EmailStr] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    date_of_birth: Optional[date] = None
    address: Optional[str] = None
    national_code: Optional[str] = None
    about: Optional[str] = None

    @field_validator("date_of_birth", mode="after")
    def convert_dob_to_str(cls, v):
        return str(v) if v else None


class Profile(BaseModel):
    job: Optional[str]
    last_education_degree: Optional[str]
    major: Optional[str]
    father_name: Optional[str]
    first_name_en: Optional[str]
    last_name_en: Optional[str]
    marital_status: Optional[str]
    gender: Optional[str]
    mobile2: Optional[str]
    landline: Optional[str]
    email2: Optional[EmailStr]
    country: Optional[str]
    province: Optional[str]
    city: Optional[str]
    programming_interests: Optional[str]


class WorkExperience(BaseModel):
    programming_languages: Optional[str]
    programming_tools: Optional[str]
    summary: Optional[str]
    programming_level: Optional[str]


class Skill(BaseModel):
    problem_solving: Optional[int]
    technology_familiarity: Optional[int]


class EducationalExperience(BaseModel):
    previous_courses: Optional[str]
    feedback: Optional[str]
    learning_challenges: Optional[str]
    course_selection_criteria: Optional[str]


class Goal(BaseModel):
    course_discovery_method: Optional[str]
    course_participation_goal: Optional[str]
    daily_study_hours: Optional[str]
    preferred_device: Optional[str]
    preferred_learning_format: Optional[str]


class Suggestion(BaseModel):
    questions_or_notes: Optional[str]
    vip_interest: Optional[bool]
    future_course_suggestions: Optional[str]


class SocialMedia(BaseModel):
    active_social_media: Optional[str]
    notification_preference: Optional[str]
    online_profile: Optional[str]


class Transaction(BaseModel):
    title: str
    pay_amount: int
    credit_amount: int
    transaction_type: Literal["add", "spend", "gift"]
    status: Literal["success", "failed"]
    created_at: datetime


class Wallet(BaseModel):
    uuid_id: str
    owner_id: str
    inventory: int
    transactions: List[Transaction]

    @model_validator(mode="before")
    def fill_owner_id(cls, values):
        # if owner_id not provided directly, look for owner_identifier in input
        if values.get("owner_identifier"):
            values["owner_id"] = values["owner_identifier"]
        return values


class WalletPackage(BaseModel):
    pay_amount: int
    credit_amount: int
    gift_percentage: int


class WalletPackages(WalletPackage):
    name: str



import asyncio
import json
import signal
import time
from typing import List, Optional

from grpc.aio import Channel, insecure_channel
from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict


class GRPCSettings(BaseSettings):
    """Configs are done by env vars because the package is mainly singleton."""

    # Server settings
    HOST: str = "127.0.0.1"
    PORT: int = 50051
    API_KEY: str = ""

    # Shutdown settings
    CHANNEL_SHUTDOWN_GRACE: float = 10.0

    # Keepalive settings
    KEEPALIVE_TIME_MS: int = 60000  # ping every 1 min
    KEEPALIVE_TIMEOUT_MS: int = 5000  # wait 5s for ack
    KEEPALIVE_PERMIT_WITHOUT_CALLS: int = 1  # even without active RPCs
    MAX_PINGS_WITHOUT_DATA: int = 0  # unlimited pings without data

    # Retry policy settings
    # We prefer aiming for only server glitches; restarts are handled by docker swarm zero-downtime
    MAX_ATTEMPTS: int = 5  # 1 original + 4 retries
    INITIAL_BACKOFF: str = "0.1s"  # quick first retry
    MAX_BACKOFF: str = "1.5s"  # max wait 1.5 second
    BACKOFF_MULTIPLIER: float = 2.0  # double wait time each retry
    RETRYABLE_STATUS_CODES: List[str] = Field(
        default=[
            "UNAVAILABLE",  # server temporarily unreachable
            "RESOURCE_EXHAUSTED",  # too many requests (try later)
            "ABORTED",  # conflict or retryable error
        ]
    )

    # RPC level configs
    WAIT_FOR_READY: bool = True
    TIMEOUT: Optional[float] = 5.0  # seconds

    model_config = SettingsConfigDict(env_prefix="ASYNC_ACCOUNT_SDK_")


grpc_settings = GRPCSettings()

SERVICE_NAME = "ADMAC"
TARGET = f"{grpc_settings.HOST}:{grpc_settings.PORT}"
CHANNEL: Channel = None

KEEPALIVE_OPTS = [
    ("grpc.keepalive_time_ms", grpc_settings.KEEPALIVE_TIME_MS),
    ("grpc.keepalive_timeout_ms", grpc_settings.KEEPALIVE_TIMEOUT_MS),
    (
        "grpc.keepalive_permit_without_calls",
        grpc_settings.KEEPALIVE_PERMIT_WITHOUT_CALLS,
    ),
    ("grpc.http2.max_pings_without_data", grpc_settings.MAX_PINGS_WITHOUT_DATA),
]

RETRY_CONFIG = {
    "methodConfig": [
        {
            "name": [{}],  # All methods
            "retryPolicy": {
                "maxAttempts": grpc_settings.MAX_ATTEMPTS,
                "initialBackoff": grpc_settings.INITIAL_BACKOFF,
                "maxBackoff": grpc_settings.MAX_BACKOFF,
                "backoffMultiplier": grpc_settings.BACKOFF_MULTIPLIER,
                "retryableStatusCodes": grpc_settings.RETRYABLE_STATUS_CODES,
            },
        }
    ]
}


async def construct_channel():
    global CHANNEL
    CHANNEL = insecure_channel(
        TARGET,
        options=KEEPALIVE_OPTS + [("grpc.service_config", json.dumps(RETRY_CONFIG))],
    )

    async def graceful_shutdown():
        print(
            f"SIGTERM received, shutting down the {SERVICE_NAME} channel gracefully..."
        )
        start_time = time.time()

        try:
            await CHANNEL.close(grace=grpc_settings.CHANNEL_SHUTDOWN_GRACE)
        except Exception as e:
            print(f"Error during shutdown: {e}")

        elapsed_time = time.time() - start_time
        print(
            f"{SERVICE_NAME} channel gracefully shut down in {elapsed_time:.2f} seconds."
        )

    # We want the currently sent requests to get completed before termination
    loop = asyncio.get_running_loop()
    loop.add_signal_handler(
        signal.SIGTERM, lambda: asyncio.create_task(graceful_shutdown())
    )
    print(f"Graceful shutdown on sigterm singal registered for {SERVICE_NAME} channel")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: accounts/grpc/accounts.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1c\x61\x63\x63ounts/grpc/accounts.proto\x12\x10\x64\x61rsman.accounts\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/protobuf/struct.proto\"\xeb\x02\n\x12\x41ssetCreateRequest\x12\x10\n\x08owner_id\x18\x01 \x01(\t\x12\x10\n\x08\x61sset_id\x18\x02 \x01(\t\x12\x16\n\tis_active\x18\x03 \x01(\x08H\x00\x88\x01\x01\x12\x18\n\x0b\x65xpire_date\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\x0c\n\x04type\x18\x05 \x01(\t\x12\x12\n\nasset_name\x18\x06 \x01(\t\x12\x30\n\nextra_data\x18\x07 \x01(\x0b\x32\x17.google.protobuf.StructH\x02\x88\x01\x01\x12#\n\x16installment_identifier\x18\x08 \x01(\tH\x03\x88\x01\x01\x12#\n\x16registration_completed\x18\t \x01(\x08H\x04\x88\x01\x01\x42\x0c\n\n_is_activeB\x0e\n\x0c_expire_dateB\r\n\x0b_extra_dataB\x19\n\x17_installment_identifierB\x19\n\x17_registration_completed\"\xf6\x01\n\x1c\x45\x64ucationalExperienceRequest\x12\x1d\n\x10previous_courses\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08\x66\x65\x65\x64\x62\x61\x63k\x18\x02 \x01(\tH\x01\x88\x01\x01\x12 \n\x13learning_challenges\x18\x03 \x01(\tH\x02\x88\x01\x01\x12&\n\x19\x63ourse_selection_criteria\x18\x04 \x01(\tH\x03\x88\x01\x01\x42\x13\n\x11_previous_coursesB\x0b\n\t_feedbackB\x16\n\x14_learning_challengesB\x1c\n\x1a_course_selection_criteria\"\xf7\x01\n\x1d\x45\x64ucationalExperienceResponse\x12\x1d\n\x10previous_courses\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08\x66\x65\x65\x64\x62\x61\x63k\x18\x02 \x01(\tH\x01\x88\x01\x01\x12 \n\x13learning_challenges\x18\x03 \x01(\tH\x02\x88\x01\x01\x12&\n\x19\x63ourse_selection_criteria\x18\x04 \x01(\tH\x03\x88\x01\x01\x42\x13\n\x11_previous_coursesB\x0b\n\t_feedbackB\x16\n\x14_learning_challengesB\x1c\n\x1a_course_selection_criteria\"\xc5\x02\n\x0bGoalRequest\x12$\n\x17\x63ourse_discovery_method\x18\x01 \x01(\tH\x00\x88\x01\x01\x12&\n\x19\x63ourse_participation_goal\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1e\n\x11\x64\x61ily_study_hours\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1d\n\x10preferred_device\x18\x04 \x01(\tH\x03\x88\x01\x01\x12&\n\x19preferred_learning_format\x18\x05 \x01(\tH\x04\x88\x01\x01\x42\x1a\n\x18_course_discovery_methodB\x1c\n\x1a_course_participation_goalB\x14\n\x12_daily_study_hoursB\x13\n\x11_preferred_deviceB\x1c\n\x1a_preferred_learning_format\"\xc6\x02\n\x0cGoalResponse\x12$\n\x17\x63ourse_discovery_method\x18\x01 \x01(\tH\x00\x88\x01\x01\x12&\n\x19\x63ourse_participation_goal\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1e\n\x11\x64\x61ily_study_hours\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1d\n\x10preferred_device\x18\x04 \x01(\tH\x03\x88\x01\x01\x12&\n\x19preferred_learning_format\x18\x05 \x01(\tH\x04\x88\x01\x01\x42\x1a\n\x18_course_discovery_methodB\x1c\n\x1a_course_participation_goalB\x14\n\x12_daily_study_hoursB\x13\n\x11_preferred_deviceB\x1c\n\x1a_preferred_learning_format\"$\n\x10ListAssetRequest\x12\x10\n\x08owner_id\x18\x01 \x01(\t\"\xe0\x04\n\x0eProfileRequest\x12\x10\n\x03job\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\"\n\x15last_education_degree\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x12\n\x05major\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x18\n\x0b\x66\x61ther_name\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1a\n\rfirst_name_en\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x19\n\x0clast_name_en\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x1b\n\x0emarital_status\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x13\n\x06gender\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x14\n\x07mobile2\x18\t \x01(\tH\x08\x88\x01\x01\x12\x15\n\x08landline\x18\n \x01(\tH\t\x88\x01\x01\x12\x13\n\x06\x65mail2\x18\x0b \x01(\tH\n\x88\x01\x01\x12\x14\n\x07\x63ountry\x18\x0c \x01(\tH\x0b\x88\x01\x01\x12\x15\n\x08province\x18\r \x01(\tH\x0c\x88\x01\x01\x12\x11\n\x04\x63ity\x18\x0e \x01(\tH\r\x88\x01\x01\x12\"\n\x15programming_interests\x18\x0f \x01(\tH\x0e\x88\x01\x01\x42\x06\n\x04_jobB\x18\n\x16_last_education_degreeB\x08\n\x06_majorB\x0e\n\x0c_father_nameB\x10\n\x0e_first_name_enB\x0f\n\r_last_name_enB\x11\n\x0f_marital_statusB\t\n\x07_genderB\n\n\x08_mobile2B\x0b\n\t_landlineB\t\n\x07_email2B\n\n\x08_countryB\x0b\n\t_provinceB\x07\n\x05_cityB\x18\n\x16_programming_interests\"\xe1\x04\n\x0fProfileResponse\x12\x10\n\x03job\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\"\n\x15last_education_degree\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x12\n\x05major\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x18\n\x0b\x66\x61ther_name\x18\x04 \x01(\tH\x03\x88\x01\x01\x12\x1a\n\rfirst_name_en\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x19\n\x0clast_name_en\x18\x06 \x01(\tH\x05\x88\x01\x01\x12\x1b\n\x0emarital_status\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x13\n\x06gender\x18\x08 \x01(\tH\x07\x88\x01\x01\x12\x14\n\x07mobile2\x18\t \x01(\tH\x08\x88\x01\x01\x12\x15\n\x08landline\x18\n \x01(\tH\t\x88\x01\x01\x12\x13\n\x06\x65mail2\x18\x0b \x01(\tH\n\x88\x01\x01\x12\x14\n\x07\x63ountry\x18\x0c \x01(\tH\x0b\x88\x01\x01\x12\x15\n\x08province\x18\r \x01(\tH\x0c\x88\x01\x01\x12\x11\n\x04\x63ity\x18\x0e \x01(\tH\r\x88\x01\x01\x12\"\n\x15programming_interests\x18\x0f \x01(\tH\x0e\x88\x01\x01\x42\x06\n\x04_jobB\x18\n\x16_last_education_degreeB\x08\n\x06_majorB\x0e\n\x0c_father_nameB\x10\n\x0e_first_name_enB\x0f\n\r_last_name_enB\x11\n\x0f_marital_statusB\t\n\x07_genderB\n\n\x08_mobile2B\x0b\n\t_landlineB\t\n\x07_email2B\n\n\x08_countryB\x0b\n\t_provinceB\x07\n\x05_cityB\x18\n\x16_programming_interests\"o\n\x08Response\x12\x11\n\x04\x63ode\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x14\n\x07message\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\x07\n\x05_codeB\n\n\x08_message\"\x80\x01\n\x0cSkillRequest\x12\x1c\n\x0fproblem_solving\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12#\n\x16technology_familiarity\x18\x02 \x01(\x05H\x01\x88\x01\x01\x42\x12\n\x10_problem_solvingB\x19\n\x17_technology_familiarity\"\x81\x01\n\rSkillResponse\x12\x1c\n\x0fproblem_solving\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12#\n\x16technology_familiarity\x18\x02 \x01(\x05H\x01\x88\x01\x01\x42\x12\n\x10_problem_solvingB\x19\n\x17_technology_familiarity\"\xc0\x01\n\x12SocialMediaRequest\x12 \n\x13\x61\x63tive_social_media\x18\x01 \x01(\tH\x00\x88\x01\x01\x12$\n\x17notification_preference\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1b\n\x0eonline_profile\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x16\n\x14_active_social_mediaB\x1a\n\x18_notification_preferenceB\x11\n\x0f_online_profile\"\xc1\x01\n\x13SocialMediaResponse\x12 \n\x13\x61\x63tive_social_media\x18\x01 \x01(\tH\x00\x88\x01\x01\x12$\n\x17notification_preference\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1b\n\x0eonline_profile\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x16\n\x14_active_social_mediaB\x1a\n\x18_notification_preferenceB\x11\n\x0f_online_profile\"\xbd\x01\n\x11SuggestionRequest\x12\x1f\n\x12questions_or_notes\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x0cvip_interest\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12&\n\x19\x66uture_course_suggestions\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x15\n\x13_questions_or_notesB\x0f\n\r_vip_interestB\x1c\n\x1a_future_course_suggestions\"\xbe\x01\n\x12SuggestionResponse\x12\x1f\n\x12questions_or_notes\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x0cvip_interest\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12&\n\x19\x66uture_course_suggestions\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x15\n\x13_questions_or_notesB\x0f\n\r_vip_interestB\x1c\n\x1a_future_course_suggestions\"*\n\x17UserBulkRetrieveRequest\x12\x0f\n\x07uuid_id\x18\x01 \x03(\t\"&\n\x13UserRetrieveRequest\x12\x0f\n\x07uuid_id\x18\x01 \x01(\t\"\xe4\x06\n\x11UserUpdateRequest\x12\x12\n\x05\x65mail\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x14\n\x07\x61\x64\x64ress\x18\x03 \x01(\tH\x01\x88\x01\x01\x12\x1a\n\rdate_of_birth\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\x12\n\x05\x61\x62out\x18\x05 \x01(\tH\x03\x88\x01\x01\x12\x1a\n\rnational_code\x18\x06 \x01(\tH\x04\x88\x01\x01\x12\x16\n\tlast_name\x18\x07 \x01(\tH\x05\x88\x01\x01\x12\x17\n\nfirst_name\x18\x08 \x01(\tH\x06\x88\x01\x01\x12\x36\n\x07profile\x18\t \x01(\x0b\x32 .darsman.accounts.ProfileRequestH\x07\x88\x01\x01\x12\x14\n\x07uuid_id\x18\n \x01(\tH\x08\x88\x01\x01\x12\x32\n\x05skill\x18\x0b \x01(\x0b\x32\x1e.darsman.accounts.SkillRequestH\t\x88\x01\x01\x12\x45\n\x0fwork_experience\x18\x0c \x01(\x0b\x32\'.darsman.accounts.WorkExperienceRequestH\n\x88\x01\x01\x12\x30\n\x04goal\x18\r \x01(\x0b\x32\x1d.darsman.accounts.GoalRequestH\x0b\x88\x01\x01\x12S\n\x16\x65\x64ucational_experience\x18\x0e \x01(\x0b\x32..darsman.accounts.EducationalExperienceRequestH\x0c\x88\x01\x01\x12?\n\x0csocial_media\x18\x0f \x01(\x0b\x32$.darsman.accounts.SocialMediaRequestH\r\x88\x01\x01\x12<\n\nsuggestion\x18\x10 \x01(\x0b\x32#.darsman.accounts.SuggestionRequestH\x0e\x88\x01\x01\x42\x08\n\x06_emailB\n\n\x08_addressB\x10\n\x0e_date_of_birthB\x08\n\x06_aboutB\x10\n\x0e_national_codeB\x0c\n\n_last_nameB\r\n\x0b_first_nameB\n\n\x08_profileB\n\n\x08_uuid_idB\x08\n\x06_skillB\x12\n\x10_work_experienceB\x07\n\x05_goalB\x19\n\x17_educational_experienceB\x0f\n\r_social_mediaB\r\n\x0b_suggestion\",\n\x1cWalletPackageRetrieveRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\")\n\x15WalletRetrieveRequest\x12\x10\n\x08owner_id\x18\x01 \x01(\t\"5\n\x12WalletSpendRequest\x12\x0f\n\x07uuid_id\x18\x01 \x01(\t\x12\x0e\n\x06\x61mount\x18\x02 \x01(\x05\"q\n\x12WalletTopupRequest\x12\x13\n\x06\x61mount\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12\x19\n\x0cpackage_name\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x0f\n\x07uuid_id\x18\x03 \x01(\tB\t\n\x07_amountB\x0f\n\r_package_name\"\xe3\x01\n\x15WorkExperienceRequest\x12\"\n\x15programming_languages\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1e\n\x11programming_tools\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x14\n\x07summary\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1e\n\x11programming_level\x18\x04 \x01(\tH\x03\x88\x01\x01\x42\x18\n\x16_programming_languagesB\x14\n\x12_programming_toolsB\n\n\x08_summaryB\x14\n\x12_programming_level\"\xe4\x01\n\x16WorkExperienceResponse\x12\"\n\x15programming_languages\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1e\n\x11programming_tools\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x14\n\x07summary\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x1e\n\x11programming_level\x18\x04 \x01(\tH\x03\x88\x01\x01\x42\x18\n\x16_programming_languagesB\x14\n\x12_programming_toolsB\n\n\x08_summaryB\x14\n\x12_programming_level2\xa9\x01\n\x0f\x41ssetController\x12L\n\x06\x43reate\x12$.darsman.accounts.AssetCreateRequest\x1a\x1a.darsman.accounts.Response\"\x00\x12H\n\x04List\x12\".darsman.accounts.ListAssetRequest\x1a\x1a.darsman.accounts.Response\"\x00\x32\x87\x02\n\x0eUserController\x12W\n\x0c\x42ulkRetrieve\x12).darsman.accounts.UserBulkRetrieveRequest\x1a\x1a.darsman.accounts.Response\"\x00\x12O\n\x08Retrieve\x12%.darsman.accounts.UserRetrieveRequest\x1a\x1a.darsman.accounts.Response\"\x00\x12K\n\x06Update\x12#.darsman.accounts.UserUpdateRequest\x1a\x1a.darsman.accounts.Response\"\x00\x32\xa6\x03\n\x10WalletController\x12\x44\n\x0cListPackages\x12\x16.google.protobuf.Empty\x1a\x1a.darsman.accounts.Response\"\x00\x12Q\n\x08Retrieve\x12\'.darsman.accounts.WalletRetrieveRequest\x1a\x1a.darsman.accounts.Response\"\x00\x12_\n\x0fRetrievePackage\x12..darsman.accounts.WalletPackageRetrieveRequest\x1a\x1a.darsman.accounts.Response\"\x00\x12K\n\x05Spend\x12$.darsman.accounts.WalletSpendRequest\x1a\x1a.darsman.accounts.Response\"\x00\x12K\n\x05Topup\x12$.darsman.accounts.WalletTopupRequest\x1a\x1a.darsman.accounts.Response\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'accounts.grpc.accounts_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_ASSETCREATEREQUEST']._serialized_start=110
  _globals['_ASSETCREATEREQUEST']._serialized_end=473
  _globals['_EDUCATIONALEXPERIENCEREQUEST']._serialized_start=476
  _globals['_EDUCATIONALEXPERIENCEREQUEST']._serialized_end=722
  _globals['_EDUCATIONALEXPERIENCERESPONSE']._serialized_start=725
  _globals['_EDUCATIONALEXPERIENCERESPONSE']._serialized_end=972
  _globals['_GOALREQUEST']._serialized_start=975
  _globals['_GOALREQUEST']._serialized_end=1300
  _globals['_GOALRESPONSE']._serialized_start=1303
  _globals['_GOALRESPONSE']._serialized_end=1629
  _globals['_LISTASSETREQUEST']._serialized_start=1631
  _globals['_LISTASSETREQUEST']._serialized_end=1667
  _globals['_PROFILEREQUEST']._serialized_start=1670
  _globals['_PROFILEREQUEST']._serialized_end=2278
  _globals['_PROFILERESPONSE']._serialized_start=2281
  _globals['_PROFILERESPONSE']._serialized_end=2890
  _globals['_RESPONSE']._serialized_start=2892
  _globals['_RESPONSE']._serialized_end=3003
  _globals['_SKILLREQUEST']._serialized_start=3006
  _globals['_SKILLREQUEST']._serialized_end=3134
  _globals['_SKILLRESPONSE']._serialized_start=3137
  _globals['_SKILLRESPONSE']._serialized_end=3266
  _globals['_SOCIALMEDIAREQUEST']._serialized_start=3269
  _globals['_SOCIALMEDIAREQUEST']._serialized_end=3461
  _globals['_SOCIALMEDIARESPONSE']._serialized_start=3464
  _globals['_SOCIALMEDIARESPONSE']._serialized_end=3657
  _globals['_SUGGESTIONREQUEST']._serialized_start=3660
  _globals['_SUGGESTIONREQUEST']._serialized_end=3849
  _globals['_SUGGESTIONRESPONSE']._serialized_start=3852
  _globals['_SUGGESTIONRESPONSE']._serialized_end=4042
  _globals['_USERBULKRETRIEVEREQUEST']._serialized_start=4044
  _globals['_USERBULKRETRIEVEREQUEST']._serialized_end=4086
  _globals['_USERRETRIEVEREQUEST']._serialized_start=4088
  _globals['_USERRETRIEVEREQUEST']._serialized_end=4126
  _globals['_USERUPDATEREQUEST']._serialized_start=4129
  _globals['_USERUPDATEREQUEST']._serialized_end=4997
  _globals['_WALLETPACKAGERETRIEVEREQUEST']._serialized_start=4999
  _globals['_WALLETPACKAGERETRIEVEREQUEST']._serialized_end=5043
  _globals['_WALLETRETRIEVEREQUEST']._serialized_start=5045
  _globals['_WALLETRETRIEVEREQUEST']._serialized_end=5086
  _globals['_WALLETSPENDREQUEST']._serialized_start=5088
  _globals['_WALLETSPENDREQUEST']._serialized_end=5141
  _globals['_WALLETTOPUPREQUEST']._serialized_start=5143
  _globals['_WALLETTOPUPREQUEST']._serialized_end=5256
  _globals['_WORKEXPERIENCEREQUEST']._serialized_start=5259
  _globals['_WORKEXPERIENCEREQUEST']._serialized_end=5486
  _globals['_WORKEXPERIENCERESPONSE']._serialized_start=5489
  _globals['_WORKEXPERIENCERESPONSE']._serialized_end=5717
  _globals['_ASSETCONTROLLER']._serialized_start=5720
  _globals['_ASSETCONTROLLER']._serialized_end=5889
  _globals['_USERCONTROLLER']._serialized_start=5892
  _globals['_USERCONTROLLER']._serialized_end=6155
  _globals['_WALLETCONTROLLER']._serialized_start=6158
  _globals['_WALLETCONTROLLER']._serialized_end=6580
# @@protoc_insertion_point(module_scope)



from setuptools import setup

with open("README.md", "r") as f:
    long_description = f.read()


setup(
    name="admac",
    version="1.2.1",
    author="Ellnamin",
    author_email="ellnamin.business@gmail.com",
    description="Async Account Service for Darsman Academy",
    long_description=long_description,
    packages=["admac", "admac/grpc"],
    install_requires=[
        "annotated-types==0.6.0",
        "email-validator==2.1.1",
        "grpcio==1.62.1",
        "grpcio-tools==1.62.1",
        "protobuf==4.25.3",
        "pydantic==2.7.0",
        "pydantic-settings==2.10.1",
        "pydantic_core==2.18.1",
        "python-dotenv==1.1.1",
        "typing-inspection==0.4.1",
        "typing_extensions==4.14.1",
    ],
    setup_requires=["wheel", "setuptools"],
)
